/*********************************************************/
/* This file is part of strs a statistical test suite    */
/*              of random sequences.                     */
/*             David Lubicz CELAR                        */
/*            Copyright (C) 2006 David Lubicz            */
/*********************************************************/

/*
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include <string.h>
#include "../include/statistic_tests.h"
#include "../include/special_functions.h"

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
                          SPECTRAL TEST
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

long int F(bsize_t x, void *parameter)

{
  bsize_t *tab;
  long int result;

  tab= (bsize_t *)parameter;

  result = (long int)tab[x];
  return result;
}

/*
proto: bsize_t *Spectral_Sample(struct BitField *Field, int order)
type: low 
comment: This function computes the spectal test for struct BitField *Field.
*/
bsize_t *Spectral_Sample(struct BitField *Field, int order)

{
  word_t i;
  bsize_t *array;
  word_t k;

    if (order >= WORDLENGTH || order==0) 
    return 0;

    array = (bsize_t *) calloc(1<<order,sizeof(bsize_t));
    
    for(i=1; i <= BITLENGTH(Field)-order; i = i+1)
      {
	k = BF_Take(Field,i) << (WORDLENGTH - order) >> (WORDLENGTH - order); 
	array[k]=array[k]+1;
      }
    return array;
}

/*
proto: struct test Statistic_Spectral(struct test_parameters parameters)
type: high
comment: This function computes the KolmogorovSmirvov test for the Spectral test
*/
struct test Statistic_Spectral(struct test_parameters parameters)

{
  struct test result;
  struct statistic chi2;
  struct BitField *array;
  double *proba, *probamem;
  bsize_t **stat_array, offset, j,i,k, stat_pattern[1], bitlength, mempattern;
  word_t bword;
  unsigned int nliberty;
  double KS, min, max, minmem, maxmem, KSmax, KSmoy;
  int flag;
  long int fourrier;
  

  if ((proba = (double *)calloc(parameters.nb_chunks, sizeof(double))) && (probamem=(double *)calloc(parameters.nb_chunks, sizeof(double))))
    {
      /* hashing the sequence of bit into chunks of the same size to pass a Chi 2 test*/
      array = BF_Split(parameters.data,parameters.nb_chunks);

#ifdef STATISTICS
      printf("Number of chunks : %d \n", parameters.nb_chunks);
      printf("Length of a chunk in bits : %d \n", BITLENGTH((&array[0])));
#endif

      stat_array = (bsize_t **)calloc(parameters.nb_chunks, sizeof(bsize_t *));
      
      /* collecting statistics */
      for(i=0; i<parameters.nb_chunks; i++)
	{
	  stat_array[i]=Spectral_Sample(&array[i], parameters.length);
	}

      KSmax=0; KSmoy=0;
      /* this loop to compute a Chi2 for each pattern */
      for(i=0; i<=(1<<(parameters.length-1))-1; i++)
	{
	  /* for each chunk compute the chi2 statistic with respect to the pattern i*/
	  min=HUGE_VAL;
	  max=-HUGE_VAL;

	  for(j=0; j<parameters.nb_chunks; j++)
	    {
	      bword=1+(i<<1);
	      fourrier=Fourrier_Word((long int(*)(bsize_t, void *))F, bword, parameters.length, stat_array[j]);
	      bitlength =BITLENGTH((&array[j]))-parameters.length+1;
	      stat_pattern[0]=(bsize_t)(bitlength-fourrier)/2;
	      stat_pattern[1]=(bsize_t)(bitlength-stat_pattern[0]);
	      chi2=Equi_Chi_Carre(stat_pattern,1);
	      proba[j]=chi2.z;
	      if (min > proba[j])
		min = proba[j];
	      if (max < proba[j])
		max = proba[j];
	    }
	  nliberty=1;
	  KS = Kolmogorov_Smirnov(proba, parameters.nb_chunks, (double (*)(double, void *))chi2_distribution, &nliberty);

#if STATISTICS >= 3
	  printf("With pattern %d, KS = %f\n", i, KS);
#endif 
	  
	  KSmoy = KSmoy + KS;

	  if (KS>KSmax)
	    {
	      KSmax=KS;
	      for(k=0; k<parameters.nb_chunks; k++)
		{
		  probamem[k]=proba[k];
		}
	      mempattern = 1+(i<<1);
	      minmem=min;
	      maxmem=max;
	    }
	}
      
      KSmoy = KSmoy / (1<<(parameters.length-1));

      min = MIN(parameters.min,minmem);
      max = MAX(parameters.max,maxmem);

      if (strcmp(parameters.file_name,"") != 0)
	{
	  plot_graph(probamem, parameters.nb_chunks, parameters.file_name);
	}
      if (strcmp(parameters.file_name1,"") != 0)
	{
      	 plot_function((double (*)(double, void *))chi2_distribution, &nliberty, min, max, parameters.nb_points, parameters.file_name1);
	}
      
      /* freeing memory */
      free(proba);
      free(probamem);
      offset=BITLENGTH(parameters.data)%parameters.nb_chunks;
      for(i=0; i<parameters.nb_chunks+(offset != 0); i++)
	{
	  BF_Dispose(&array[i]);
	}
      free(array);
      for(i=0; i<parameters.nb_chunks; i++)
	{
	  free(stat_array[i]);
	}
      free(stat_array);
      
      /* formating the result */
      result.min = minmem;
      result.max = maxmem;
      result.z= KSmoy;
      result.error=0;
      result.warning = flag;
      result.parameter = mempattern;
      return result;
    }
  else
    {
      result.error=1;
      result.z=0;
      return result;
    }
}
