/*********************************************************/
/* This file is part of strs a statistical test suite    */
/*              of random sequences.                     */
/*             David Lubicz CELAR                        */
/*            Copyright (C) 2006 David Lubicz            */
/*********************************************************/

/*
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include <string.h>
#include "../include/test.h"
#include "../include/statistic_tests.h"
#include "../include/special_functions.h"

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
                          M A U R E R      T E S T
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/*
proto: struct statistic Maurer(struct BitField *Field, int length)
type: low 
comment: This function computes the Maurer test for words of length given by the parameter length
*/
struct statistic Maurer(struct BitField *Field, int length)
{
  bsize_t i, j, *array, compteur;
  word_t k;
  struct statistic result;
  double sigma, sigma1,clk;
  
  if (length >= MAURER_MAX || length==0)
    {
      result.error=1;
    return result;
    }

  array = (bsize_t *) calloc(1<<length,sizeof(bsize_t));
  
  compteur=1<<length;
  i=0;
  while ((compteur>0) && ((1+i*length)<BITLENGTH(Field)-WORDLENGTH))
  {
    k = BF_Take(Field,1+i*length) << (WORDLENGTH - length) >> (WORDLENGTH - length);
    if (array[k]==0)
      {
      compteur--;
      }
    array[k]=i;
    i++;
  }

  if (compteur==0)
    {
      result.z = 0;
      for(j=i; (1+j*length)<BITLENGTH(Field)-WORDLENGTH; j++)
	{
	  k = BF_Take(Field,1+j*length) << (WORDLENGTH - length) >> (WORDLENGTH - length);
	  result.z = result.z + log((double)(j-array[k]))/log(2);
	  array[k]=j;
	}
      result.z = result.z/ (double)(j-i);

      sigma1=Maurer_Data[length-1][1];
      clk = 0.7-(0.8/(double)length)+(1.6+(12.8/(double)length))*pow(j-i,-4.0/(double)length);
      sigma = sigma1/(double)(j-i)*pow(clk,2);
      result.z=(double)(result.z - Maurer_Data[length-1][0])/sqrt(sigma);

#if STATISTICS >= 1
      printf("=======> %f \n", result.z);
#endif

    }
  else
    result.error=1;
  

  free(array);


  return result;
}

/*
proto: struct test Statistic_Maurer(struct test_parameters parameters)
type: high
comment: This function computes the KolmogorovSmirnov test for the Maurer test
*/
struct test Statistic_Maurer(struct test_parameters parameters)
{
  struct test result;
  struct statistic chi2;
  struct BitField *array;
  double *proba;
  unsigned int i;
  double KS, min, max;
  bsize_t offset;
  int flag, parameter;

  if ((proba = (double *)calloc(parameters.nb_chunks, sizeof(double))) && (parameters.length <= MAURER_MAX) && (parameters.length > 0))
    {
      array = BF_Split(parameters.data,parameters.nb_chunks);

#ifdef STATISTICS
      printf("Number of chunks : %d \n", parameters.nb_chunks);
      printf("Length of a chunk in bits : %d \n", BITLENGTH((&array[0])));
#endif

      min = HUGE_VAL; 
      max = -HUGE_VAL;
      flag=0;
      for(i=0; i<parameters.nb_chunks; i++)
	{
	  chi2=Maurer(&array[i],parameters.length);

#if STATISTICS >= 1
	  printf("Chunks %d, chi2 = %f\n", i, chi2.z);
#endif 
	  
	  proba[i]=chi2.z;
	  flag= chi2.warning | flag;
	  if (min > proba[i])
	    min = proba[i];
	  if (max < proba[i])
	    max = proba[i];
	}
      KS = Kolmogorov_Smirnov(proba, parameters.nb_chunks, (double (*)(double, void *))normal, &parameter);

      min = MIN(parameters.min,min);
      max = MAX(parameters.max,max);

      if (strcmp(parameters.file_name,"") != 0)
	{
	  plot_graph(proba, parameters.nb_chunks, parameters.file_name);
	}
      if (strcmp(parameters.file_name1,"") != 0)
	{
	  plot_function((double (*)(double, void *))normal, &parameter, min, max, parameters.nb_points, parameters.file_name1);
	}


      free(proba);
      offset=BITLENGTH(parameters.data)%parameters.nb_chunks;
      for(i=0; i<parameters.nb_chunks+(offset != 0); i++)
	{
	  BF_Dispose(&array[i]);
	}
      free(array);
      result.min = min;
      result.max = max;
      result.z= KS;
      result.error=0;
      result.warning = flag;
      return result;
    }
  else
    {
      result.error=1;
      result.z=0;
      return result;
    }
}
