/*********************************************************/
/* This file is part of strs a statistical test suite    */
/*              of random sequences.                     */
/*             David Lubicz CELAR                        */
/*            Copyright (C) 2006 David Lubicz            */
/*********************************************************/

/*
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

/***************************************************************/
/* This library contains a set of functions to perform all the */
/* bit manipulations currently used in alea testing and block  */
/* cypher simulation                                           */
/***************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include "../include/btypes.h"

/* 
proto: void word2string(word_t x, bsize_t l, char *string, int base)
type: lowlevel function
comment: This function displays the l first bits of a word in base base
*/

void word2string(word_t x, bsize_t l, char *string, int base)

{
  char c;
  char result[129];
  bsize_t i;
  int log;
  word_t y;

  switch (base)
    {
    case 2 : log=1; y=1;
      break;
    case 8 : log=3; y=7;
      break;
    case 16 : log=4; y=15;
    }

  for (i=1; i<= MIN(WORDLENGTH, l)/log; i++)
    {
      c = x & y;
      if (c<10) 
	result[i-1] = '0'+c;
      else 
	result[i-1]='a'+c-10;
      x=x>>(unsigned long)log;
    }
  result[i-1]='\0';
  strcpy(string, result);
}

/* 
proto: word_t wordcompose(word_t word1, word_t word2, bsize_t l)
type: low level function
comment: This function returns a word composed of the l first bits of word1 and of the WORDLENGTH-l bits of word2
*/
word_t wordcompose(word_t word1, word_t word2, bsize_t l)

{
  word_t result;

  word1 = word1 << (WORDLENGTH - l) >> (WORDLENGTH - l);
  word2 = word2 >> l << l;
  result = word1 | word2;

  return result;
}

/* 
proto: bsize_t wordweigth(word_t x)
type: lowlevel function
comment: This function computes 
*/
bsize_t wordweigth(word_t x)

{
  int i;
  bsize_t result;
  
  result=0;

  for (i=1; i<= WORDLENGTH; i++)
    {
      result = result + (x & 1UL);
      x = x>>1UL;
    }
  return result;
}

/*
proto: word_t wordscalar(word_t x, word_t y)
type: lowlevel function
comment: computes the scalar product of x and y
*/
word_t wordscalar(word_t x, word_t y)

{
  int i, k;
  word_t z;

  x = x & y;

  k=1 << (LOG2WORDLENGTH-1);
  for (i = 1; i<= LOG2WORDLENGTH; i++)
    {
      z = x;
      z=z >> k;
      x = (x ^ z) << (WORDLENGTH - k) >> (WORDLENGTH - k);
      k=k >> 1;
    }
  return x & 1UL;
}

/*
proto: wordbalance(word_t x)
type: lowlevel function
comment: This function returns the number of 1 minus the number of 0 of a word
*/
bsize_t wordbalance(word_t x)
{
  bsize_t z;
  z = wordweigth(x) - wordweigth(~x);
  return z;
}

/*
proto: int BF_Create(struct BitField *Field, bsize_t l, bsize_t max)
type: initialisation function
comment: This function creates a BitField of real size l bits and allocated size max bits
*/
int BF_Create(struct BitField *Field, bsize_t l, bsize_t max)
     
{
  bsize_t length, mem; 
  boffset_t offset;
 
  if (l==0)
    {
      offset = -1;
      length=0;
      mem = (max >> LOG2WORDLENGTH)+1;
    }
  else
    {
      l=l-1;
      max=max-1;
      offset = REMAINDER(l,LOG2WORDLENGTH);
      length = (l>> LOG2WORDLENGTH)+1;
      mem = (max>> LOG2WORDLENGTH)+1;
      if (mem == 1) mem = 2;
    }
  if (Field->data = (word_t *) calloc(mem, sizeof(word_t)))
    {
      Field->offset = offset;
      Field->l = length;
      Field->mem = mem;
      return 1;
    }
  else
    return 0;
}

/*
proto: int BF_Realloc(struct BitField *Field, bsize_t l)
type: initialisation function
comment: This function reallocs the memory used for a BitField without changing its real size
*/
int BF_Realloc(struct BitField *Field, bsize_t l)

{
  bsize_t mem; 
  boffset_t offset;

  if (l==0)
    {
    Field->offset = -1;
    Field->l=0;
    mem=0;
    }
  else
    {
      l=l-1;
      offset = REMAINDER(l,LOG2WORDLENGTH)-1;
      mem = (l >> LOG2WORDLENGTH)+1;
      if (mem == 1) mem = 2;
    }
  if (Field->data = (word_t *) realloc((void *)Field->data,mem << LOG2WORDLENGTH >> 3UL))
    {
  Field->mem = l;
  return 1;
    }
  else
    return 0;
}

/*
proto: word_t BF_Take(struct BitField *Field, bsize_t l)
type: set and read functions
comment: This function returns a word of a BitField of offset l
*/
word_t BF_Take(struct BitField *Field, bsize_t l)

{
  bsize_t first; 
  boffset_t offset;
  word_t result, retenue;

  l = l -1;
  first = l >> LOG2WORDLENGTH;
  offset = l ^ (first << LOG2WORDLENGTH);

  retenue= (offset == 0);
  result = Field->data[first] >> offset | Field->data[first + 1] << (WORDLENGTH - offset - retenue) << retenue;

  return result;
}

/*
proto: int BF_Put(struct BitField *Field, bsize_t l, word_t mword)
type: set and read function
comment: This function set a word in a BitField at position l
*/
int BF_Put(struct BitField *Field, bsize_t l, word_t mword)

{
  bsize_t first;
  boffset_t offset, retenue;
  

  l = l -1;
  first = l >> LOG2WORDLENGTH;
  offset = l ^ (first << LOG2WORDLENGTH);

  retenue = (offset == 0);
  Field->data[first] = Field->data[first] ^ (mword << offset);
  Field->data[first + 1] = Field->data[first + 1] ^ (mword >> (WORDLENGTH - offset - retenue) >> retenue);
}

/*
proto: void BF_Take_Field(struct BitField *Field1, struct BitField *Field2, bsize_t l, bsize_t h)
type: set and read function
comment: This function extracts a Field of a BitField of offset l and length h
*/
void BF_Take_Field(struct BitField *Field1, struct BitField *Field2, bsize_t l, bsize_t h)

{
  bsize_t i; 
  bsize_t last;

  l = MIN(l, BITLENGTH(Field1));
  h = MIN(h, BITLENGTH(Field1)-l+1)-1;

  Field2->offset = REMAINDER(h,LOG2WORDLENGTH);
  Field2->l = (h>> LOG2WORDLENGTH)+1;

  last = h >> LOG2WORDLENGTH;
  if (last >= 1)
    {
      for (i=0; i <= last-1; i++)
	{
	  Field2->data[i]=BF_Take(Field1, (i << LOG2WORDLENGTH)+l);
	}
    }
  Field2->data[last]=BF_Take(Field1, (last << LOG2WORDLENGTH)+l);
}

/*
proto: struct BitField *BF_Split(struct BitField *Field, int n)
type: set and read function
comment: function which split a BitField into n BitField of the same length plus possibly one more of leatest length
*/
struct BitField *BF_Split(struct BitField *Field, int n)

{
  int i;
  bsize_t length, offset;
  struct BitField *array;

  array=0;
  offset=BITLENGTH(Field)%n;
  if (array=(struct BitField *)calloc(n+(offset != 0), sizeof(struct BitField)))
    {
      if (n>BITLENGTH(Field))
	n = BITLENGTH(Field);
      length = (BITLENGTH(Field)/(8*n))*8;
      for (i=0; i<n; i++)
	{
	  BF_Create(&array[i],length,length);
	  BF_Take_Field(Field,&array[i],i*length+1, length);
	}
      
      if (offset !=0)
	{
	  BF_Create(&array[n], offset, offset);
	  BF_Take_Field(Field,&array[n],n*length, offset);
	}
    }
  return array;
}

/*
proto: void BF_Append_Bit(struct BitField *Field, word_t bit)
type: read and set function
comment: This function appends a bit to a BitField
*/
void BF_Append_Bit(struct BitField *Field, word_t bit)

{
  bsize_t l;
  boffset_t offset;

  Field->offset++;
  Field->offset = Field->offset ^ (Field->offset >> LOG2WORDLENGTH << LOG2WORDLENGTH);  
  Field->l = Field->l + (Field->offset == 0);
  l = Field->l-1;
  offset = Field->offset;
  Field->data[l] = Field->data[l] << (WORDLENGTH-offset) >> (WORDLENGTH-offset) | (bit << offset);
}

/*
proto: int BF_Open_Text_File(struct BitField *Field, char *bitfile, int base)
type: input output
comment: function to read a BitField from a text file in base int base. The base can be 2, 8 or 16.
*/
int BF_Open_Text_File(struct BitField *Field, char *bitfile, int base)

{
  FILE *input;
  int flag, log,c,i, compteur;
  struct stat statistiques;


  switch (base)
    {
    case 2 : log=1;
      break;
    case 8 : log=3;
      break;
    case 16 : log=4;
    }

  stat(bitfile, &statistiques);
  BF_Create(Field, 0, statistiques.st_size*log);

  if ((input = fopen(bitfile,"r")))
    {
      while ( (c = fgetc(input)) != EOF)
	{
	  flag =0;
	  if (c >= '0' && c <= '9' && (c-'0') < base)
	    {
	      c = c - '0';
	      flag =1;
	    }
	  else
	    {
	      if (c >= 'A' && c <= 'F' && base == 16)
		{
		c = c-'A'+10;
		flag =1;
		}
	      if (c >= 'a' && c <= 'f' && base == 16)
		{
		c = c-'a'+10;
		flag =1;
		}
	    }
	  
	  if (flag == 1)
	    {
	      for (i=1; i<=log; i++)
		{
		  switch (c & 1UL) 
		    {
		    case 0 : BF_Append_Bit(Field, 0UL);
		      break;
		    case 1 : BF_Append_Bit(Field, 1UL);
		    }
		  c = c>>1;
		}
	    }
	}
        fclose(input);
    }
  return 1;
}

/*
proto: int BF_Open_File(struct BitField *Field, char *bitfile)
type: input output
comment: This function read a BitField from a binary file.
*/
int BF_Open_File(struct BitField *Field, char *bitfile)

{
  FILE *input;
  struct stat statistiques;
  bsize_t i;
  char *table;
  int c;

  stat(bitfile, &statistiques);
  BF_Create(Field, statistiques.st_size << 3UL,statistiques.st_size << 3UL);
  
  if (input = fopen(bitfile,"r"))
    {
      table = (char *) Field->data;
      i=0;
      c = fgetc(input);
      while (c != EOF)
	{
	  table[i]=c;
	  i++;
	c =fgetc(input);
	}
      fclose(input);
      return 1;
    }
  else
    return 0;
}

/*
proto: int BF_Write_File(struct BitField *Field, char *bitfile)
type: intput output
comment: This function dump a raw BitField into the file named char *bitfile
*/
int BF_Write_File(struct BitField *Field, char *bitfile)

{
  FILE *output;
  bsize_t i;
  char *table;

  if (output = fopen(bitfile,"w"))
    {
      table = (char *) Field->data;
      for (i=0; i <= BYTELENGTH(Field) - 1; i++)
	{
	  fputc(table[i], output);
	}
        fclose(output);
	return 1;
    }
  else
    return 0;
}

/*
proto: void BF_Dispose (struct BitField *Field)
type: memory function
comment: This function releases the resources devoted to a BitField
*/
void BF_Dispose (struct BitField *Field)

{
  free(Field->data);
  Field->l=0;
  Field->mem=0;
  Field->offset=-1;
}

/*
proto: void BF_2_String(struct BitField *Field, bsize_t l, char *string, int base)
type: input output
comment: function to translate a BitField in a string representing its value in base base
*/
void BF_2_String(struct BitField *Field, bsize_t l, char *string, int base)

{
  int i;
  boffset_t offset;
  char result[255];
  
  strcpy(string,"");
  if (l > BITLENGTH(Field))
    l = BITLENGTH(Field);
  
  if ((l >> LOG2WORDLENGTH) >= 1)
    {
      for (i=0; i <= (l >> LOG2WORDLENGTH) -1; i++)
	{
	  word2string(Field->data[i], WORDLENGTH, (char *)&result, base);
	  strcat(string, result);	  
	}
    }
  offset = REMAINDER(l,LOG2WORDLENGTH);
  word2string(Field->data[Field->l -1], offset, (char *)&result, base);
  strcat(string, result);	  
}

/*
proto: void BF_Display(struct BitField *Field, bsize_t l, int base)
type: input output
comment: This function displays the l first bits of a BitField
*/
void BF_Display(struct BitField *Field, bsize_t l, int base)
     
{
  int i, log;
  char buffer[255];

  switch (base)
    {
    case 2 : log=1;
      break;
    case 8 : log=3;
      break;
    case 16 : log=4;
    }

  if (l > BITLENGTH(Field))
    l = BITLENGTH(Field);

  for (i=0; i<l/(255*log); i++)
    {
      BF_2_String(Field, 255*log, (char *)&buffer, base);
      printf("%s",buffer);
    }
  BF_2_String(Field, l%(255*log), (char *)&buffer, base);
  printf("%s", buffer);
  printf("\n");
}

/*
proto: void BF_FDisplay(struct BitField *Field, bsize_t l, int base, FILE *output)
type: input output
comment: This function write to a file the l first bits of a BitField */
void BF_FDisplay(struct BitField *Field, bsize_t l, int base, FILE *output)
     
{
  int i, log;
  char buffer[255];

  switch (base)
    {
    case 2 : log=1;
      break;
    case 8 : log=3;
      break;
    case 16 : log=4;
    }

  if (l > BITLENGTH(Field))
    l = BITLENGTH(Field);

  for (i=0; i<l/(255*log); i++)
    {
      BF_2_String(Field, 255*log, (char *)&buffer, base);
      fprintf(output, "%s",buffer);
    }
  BF_2_String(Field, l%(255*log), (char *)&buffer, base);
  fprintf(output, "%s", buffer);
  fprintf(output, "\n");
}

/*
proto: void BF_Set_Bit (struct BitField *Field, bsize_t l, int mbit)
type: read and set function
comment: This function sets a bit of offset l to the value given by bit
*/
void BF_Set_Bit (struct BitField *Field, bsize_t l, int mbit)

{
  bsize_t lword;
  boffset_t offset, offset1,retenue1,retenue2;
  word_t data;
  
  l--;
  lword = l >> LOG2WORDLENGTH;
  offset = l ^ (lword << LOG2WORDLENGTH);

  mbit = (mbit & 1UL) << offset;
  data = Field->data[lword];
  
  offset1= offset +1;
  retenue1 = (offset1 == WORDLENGTH);
  offset1 = offset1 - retenue1;
  retenue2=(offset==0);
  Field->data[lword]= data >> offset1 >> retenue1 << offset1 << retenue1 | mbit | data << (WORDLENGTH - offset-retenue2) << retenue2 >> (WORDLENGTH - offset - retenue2) >> retenue2;

}

/*
proto: long int Fourrier_Word(long int (*f)(bsize_t, void *), unsigned int u, bsize_t l, void *parameter)
type: general purpose binary functions
comment: This function computes the Walsh transform of the function f evaluated with the l first bits of u
*/
long int Fourrier_Word(long int (*f)(bsize_t, void *), unsigned int u, bsize_t l, void *parameter)

{
  long int result,r;
  bsize_t i;
  bsize_t scalar;

  result = 0;
  for (i=0; i <= (1UL << l)-1; i++)
    {
      scalar = wordscalar(u,i);
      r = (*f)(i, parameter);
      result= result + r* (((long int)(scalar<<1)) - 1);
    }
  return result;
}

/*
proto: bsize_t BF_Weight(struct BitField *Field, bsize_t l)
type: general purpose binary functions
comment: This function returns the weight of the l fist bits of a BitField
*/
bsize_t BF_Weight(struct BitField *Field, bsize_t l)
     
{
  bsize_t i, result;
  boffset_t offset; 
  
  if (l > BITLENGTH(Field)) 
    l = BITLENGTH(Field);
  
  offset = REMAINDER(l, LOG2WORDLENGTH);

  if (offset != 0)
    result = wordweigth(Field->data[l >> LOG2WORDLENGTH] << (WORDLENGTH - offset));

  i = 0; result = 0;
  while (i < l >> LOG2WORDLENGTH)
    {
      result = result + wordweigth(Field->data[i]);
      i++;
    }
  return result;
}    

/*
proto: void BF_Right_Shift(struct BitField *Field, word_t mbit)
type: general purpose binary functions
comment: This function shifts to the right a BitField and complete with the value of the int
*/
void BF_Right_Shift(struct BitField *Field, word_t mbit)

{
  bsize_t l,i,j;
  word_t retenue, retenue1;

  retenue = mbit << (Field->offset);

  for (i=Field->l; i > 0; i--)
    {
      j=i-1;
      retenue1 = Field->data[j] & 1UL;
      retenue1 = retenue1 << (WORDLENGTH -1UL);
      Field->data[j] = Field->data[j] >> 1UL;
      Field->data[j] = Field->data[j] | retenue;
      retenue = retenue1;
    }

  /*  Field->data[l-1] = RSETBIT(Field->data[l], Field->offset, mbit);   */
}

/*
proto: void BF_Left_Shift(struct BitField *Field, word_t mbit)
type: general purpose binary function
comment: This function shifts to the left a BitField and complete with the value of the int
*/
void BF_Left_Shift(struct BitField *Field, word_t mbit)

{
  word_t retenue, retenue1, mask;
  bsize_t i;

  retenue = mbit & 1UL;
  mask = 1 << (WORDLENGTH -1);
  for (i=0; i<= Field->l - 1; i++)
    {
      retenue1= Field->data[i] & mask;
      retenue1 = retenue1 >> (WORDLENGTH -1);
      Field->data[i] = Field->data[i] << 1UL;
      Field->data[i] = Field->data[i] | retenue;
      retenue = retenue1;
    }
}

/*
proto: word_t BF_Take_Bit(struct BitField *Field, bsize_t l)
type: set and read functions
comment: This function returns the bit of a BitField of offset l
*/
word_t BF_Take_Bit(struct BitField *Field, bsize_t l)

{
  word_t result;
  
  l = l -1;
  result = Field->data[l >> LOG2WORDLENGTH] >> (REMAINDER(l, LOG2WORDLENGTH)) << (WORDLENGTH -1) >> (WORDLENGTH - 1);
  return result;
}

/*
proto: void BF_Reverse(struct BitField *Field1, struct BitField *Field2)
type: general purpose binary functions
comment: function which reverse the order of a BitField
*/
void BF_Reverse(struct BitField *Field1, struct BitField *Field2)

{
  bsize_t i;

  Field2->l=0;
  Field2->offset=-1;

  for(i=BITLENGTH(Field1); i>=1; i--)
    {
      BF_Append_Bit(Field2, BF_Take_Bit(Field1,i));
    }
}

/*
proto: bsize_t BF_Match(struct BitField *Field, struct BitField *Match, int *flag)
type: general purpose binary functions
comment: This function returns the offset of a chain of bits in a BitField which match a given *struct BItFIeld *Match pattern
*/
bsize_t BF_Match(struct BitField *Field, struct BitField *Match, int *flag)

{
  bsize_t i,j,offset,dec,l;
  word_t word1,word2;
  
  *flag = 0;
  l = Match->l -1;
  for (j=0; j < BITLENGTH(Field)-BITLENGTH(Match); j++)
    {
      i=0;
      while (i < l)
      {
	if (BF_Take(Field,j+(i << LOG2WORDLENGTH))==Match->data[i]) i++;
      }
      if (i == l)
	{
	  dec = 0;
	  if (Match->offset != 0)
	    {	      
	      dec = WORDLENGTH - offset;
	    }
	  word1 = BF_Take(Field, j+(l << LOG2WORDLENGTH)) << dec;
	  word2 = Match->data[l]<< dec;
	  if (word1==word2) 
	    {
	      *flag = 1;
	      return j;
	    }   
	}
    }
return 0;
}

/*
proto: void BF_And(struct BitField *Field1, struct BitField *Field2, struct BitField *Field3, bsize_t l)
type: general purpose binary function
comment: This function computes the And of two BitFields
*/
void BF_And(struct BitField *Field1, struct BitField *Field2, struct BitField *Field3, bsize_t l)

{
  bsize_t i;

  l = MIN(Field1->l, Field2->l);
  Field3->l=l;
  
  for (i=0; i <= l; i++)
    {
      Field3->data[i] = Field1->data[i] & Field2->data[i];
    }
}

/*
proto: void BF_Or(struct BitField *Field1, struct BitField *Field2, struct BitField *Field3, bsize_t l)
type: general purpose binary function
comment: This function computes the Or of two BitFields
*/
void BF_Or(struct BitField *Field1, struct BitField *Field2, struct BitField *Field3, bsize_t l)

{
  bsize_t i;

  l = MIN(Field1->l, Field2->l);
  Field3->l=l;
  
  for (i=0; i <= l; i++)
    {
      Field3->data[i] = Field1->data[i] | Field2->data[i];
    }
}

/*
proto: void BF_Xor(struct BitField *Field1, struct BitField *Field2, struct BitField *Field3, bsize_t l)
type: general purpose binary function
comment: This compute the Xor of two BitFields
*/
void BF_Xor(struct BitField *Field1, struct BitField *Field2, struct BitField *Field3, bsize_t l)
{
  bsize_t i;

  l = MIN(Field1->l, Field2->l);
  Field3->l=l;
  
  for (i=0; i <= l; i++)
    {
      Field3->data[i] = Field1->data[i] ^ Field2->data[i];
    }
}

/*
proto: word_t BF_Scalar(struct BitField *Field1, struct BitField *Field2, bsize_t l)
type: general purpose binary function
comment: This function computes the scalar product of two BitFields
*/
word_t BF_Scalar(struct BitField *Field1, struct BitField *Field2, bsize_t l)

{
  word_t result;
  bsize_t last,i;
  boffset_t offset;

  result = 0;
  
  if ((l > BITLENGTH(Field1)) || (l > BITLENGTH(Field2)))
    {
      return 0;
    }
  else
    {
      offset = REMAINDER(l, LOG2WORDLENGTH);
      last = l >> LOG2WORDLENGTH;
      i = 0;
      
      result = wordscalar(Field1->data[last] << (WORDLENGTH-offset), Field2->data[last] << (WORDLENGTH-offset));

      while (i < last)
	{
	  result = result ^ wordscalar(Field1->data[i], Field2->data[i]);
	  i++;
	}
    }
  return result;
}

/*
proto: word_t quick_BF_Scalar(struct BitField *Field1, struct BitField *Field2, bsize_t l)
type: general purpose binary function
comment: This function computes the scalar product of two BitFields
*/
word_t quick_BF_Scalar(struct BitField *Field1, struct BitField *Field2, bsize_t l)
{
  bsize_t wordl;
  boffset_t offset;
  struct BitField *Field3, *Field4;
  word_t retenue;
  
  if ((l > BITLENGTH(Field1)) || (l > BITLENGTH(Field2)))
    {
      return -1;
    }
  else
    {
      BF_Create(Field3,l,l); BF_Create(Field4,l,l);
      BF_And(Field1, Field2, Field3, l);
      
      wordl = l >> LOG2WORDLENGTH;
      offset = l ^ wordl;
      if (offset > 0)
	retenue = wordscalar(Field3->data[wordl+1] << (WORDLENGTH - offset), 0xFFFFFFFF);
      else
	retenue = 0;
      
      while (wordl >1UL)
	{
	  wordl = wordl >>1UL;
	  if (Field3->l & 1UL)
	    retenue = retenue & wordscalar(Field3->data[l], 0xFFFFFFFF);
	  Field4->l = wordl;
	  Field4->offset = 0;
	  Field4->data = Field1->data + (l << LOG2WORDLENGTH);
	  BF_And(Field3, Field4, Field3, wordl << LOG2WORDLENGTH);
	}
      retenue = retenue & wordscalar(Field3->data[0], 0xFFFFFFFF);
    }
  return retenue;
}

/*
proto: int BF_Balance(struct BitField *Field, unsigned int l)
type: general purpose binary function
comment: This function computes the number of 1 minus the number of 0 of a BitField
*/
int BF_Balance(struct BitField *Field, unsigned int l)
{
  bsize_t i, retenue, lword;
  boffset_t offset; 
  
  l = MIN(BITLENGTH(Field) , l);
  lword = l >> LOG2WORDLENGTH;
  offset = l ^ (lword << LOG2WORDLENGTH);
  if (offset != 0)
    retenue = wordbalance(Field->data[lword] << (WORDLENGTH - offset));

  for (i=0; i<= lword-1; i++)
    {
      retenue = retenue + wordbalance(Field->data[i]);
    }
  return retenue;
}


